package com.servicemall.right.service;

import java.util.*;
import javax.annotation.PostConstruct;
import javax.annotation.Resource;

import com.servicemall.systemcommon.util.DateUtil;
import team.bangbang.common.CommonMPI;
import team.bangbang.common.data.KeyValue;
import team.bangbang.common.data.Pagination;

import com.servicemall.right.data.Coupon;
import com.servicemall.right.mapper.CouponMapper;

import org.springframework.stereotype.Service;

/**
 * 优惠权益 - Service
 *
 * @author 帮帮组
 * @version 1.0  2020-12-08
 */
@Service
public final class CouponService {
	/* 优惠权益（Coupon）Mapper */
	@Resource
	private CouponMapper _couponMapper = null;
	/* 设置static的Mapper对象，主要是为了兼顾Service层的static方法需要 */
	private static CouponMapper couponMapper = null;

	@PostConstruct
	public void init() {
		// 设置static的Mapper对象，主要是为了兼顾Service层的static方法需要
		couponMapper = _couponMapper;
	}

	/**
	 * 得到指定的优惠权益
	 *
	 * @param id
	 *			指定的权益编号
	 * @return 优惠权益
	 */
	public static Coupon getObject(Long id) {
		if(couponMapper == null) {
			return null;
		}

		// 参数校验

		if(id == null || id == 0L) {
			return null;
		}

		// 查询条件
		Coupon form = new Coupon();
		form.setId(id);

		return getObject(form, null);
	}

	/**
	 * 查询一条优惠权益，并转化为相应的POJO对象
	 *
	 * @param coupon
	 *			查询条件，不能为null
	 * @param appendix
	 *			附加限定条件
	 * @return 返回结果记录，并转化为相应的POJO对象
	 */
	public static Coupon getObject(Coupon coupon, String appendix) {
		if(couponMapper == null) {
			return null;
		}

		// 参数校验
		if(coupon == null && (appendix == null || appendix.trim().length() == 0)) {
			return null;
		}

		return couponMapper.getObject(coupon, appendix);
	}

	/**
	 * 插入一条优惠权益
	 *
	 * @param data
	 *			插入的数据，不能为null
	 * @return 1：成功 其它：失败
	 */
	public static int insert(Coupon data) {
		if(couponMapper == null) {
			return 0;
		}

		if (data.getId() == null) {

			// 返回创建的关键字的值，如果是系统自动生成的，则此处不返回
			long id = CommonMPI.generateSequenceId();
			data.setId(id);

		}

		return couponMapper.insert(data);
	}

	/**
	 * 删除优惠权益
	 *
	 * @param where
	 *			删除条件，不能为null
	 * @param appendix
	 *			附加限定条件
	 * @return 成功删除的记录数量
	 */
	public static int delete(Coupon where, String appendix) {
		if(couponMapper == null) {
			return 0;
		}

		return couponMapper.delete(where, appendix);
	}

	/**
	 * 修改优惠权益
	 *
	 * @param where
	 *			更新条件，不能为null
	 * @param appendix
	 *			附加限定条件
	 * @param data
	 *			更新数据，不能为null
	 * @return 成功修改的记录数量
	 */
	public static int update(Coupon where, String appendix, Coupon data) {
		if(couponMapper == null) {
			return 0;
		}

		return couponMapper.update(where, appendix, data);
	}

	/**
	 * 查询多条优惠权益，并转化为相应的POJO对象列表
	 *
	 * @param where
	 *			更新条件，不能为null
	 * @param appendix
	 *			附加限定条件
	 * @param pagination
	 *			分页参数，如果分页参数为空，表示不分页
	 * @return 返回结果记录，并转化为相应的POJO对象列表
	 */
	public static List<Coupon> list(Coupon where, String appendix, Pagination pagination) {
		if(couponMapper == null) {
			return Collections.emptyList();
		}

		return couponMapper.list(where, appendix, pagination);
	}

	/**
	 * 获得符合条件的优惠权益数量
	 *
	 * @param where
	 *			查询条件，不能为null
	 * @param appendix
	 *			附加限定条件
	 * @return 返回记录数量
	 */
	public static int count(Coupon where, String appendix) {
		if(couponMapper == null) {
			return 0;
		}

		return couponMapper.count(where, appendix);
	}

	/**
	 * 获得有效期类型{1：按日期期间 2：按有效期}列表
	 *
	 * @return 有效期类型{1：按日期期间 2：按有效期}列表
	 */
	public static List<KeyValue> getActiveTypeList() {
		return CommonMPI.getDictionaryList(Coupon.activeTypeFlags);
	}

	/**
	 * 获得核销类型{1：订单核销 2：快速核销}列表
	 *
	 * @return 核销类型{1：订单核销 2：快速核销}列表
	 */
	public static List<KeyValue> getUsedTypeList() {
		return CommonMPI.getDictionaryList(Coupon.usedTypeFlags);
	}

	/**
	 * 获得是否可核销多次{1:是 2:否}列表
	 *
	 * @return 是否可核销多次{1:是 2:否}列表
	 */
	public static List<KeyValue> getUseNumTypeList() {
		return CommonMPI.getDictionaryList(Coupon.useNumTypeFlags);
	}

	/**
	 * 获得每{1:不限制 2:年 3:季度 4:月}列表
	 *
	 * @return 每{1:不限制 2:年 3:季度 4:月}列表
	 */
	public static List<KeyValue> getPhaseTypeList() {
		return CommonMPI.getDictionaryList(Coupon.phaseTypeFlags);
	}

	/**
	 * 获得权益类型{1:满减券金额类-整单减2:满减券金额类-每满减3:满减券产品类-整单减4:满减券产品类-每满减 5:折扣券6赠品券7免单券8代金券9动态折扣券}列表
	 *
	 * @return 权益类型{1:满减券金额类-整单减2:满减券金额类-每满减3:满减券产品类-整单减4:满减券产品类-每满减 5:折扣券6赠品券7免单券8代金券9动态折扣券}列表
	 */
	public static List<KeyValue> getRightTypeList() {
		return CommonMPI.getDictionaryList(Coupon.rightTypeFlags);
	}

	public static List<Coupon> queryCouponList(String dms_code,Pagination pagination){
		List<Coupon> couponList = new ArrayList<>();
		Coupon coupon = new Coupon();
		coupon.setActiveEndTime(DateUtil.getTime());
		coupon.setActiveStartTime(DateUtil.getTime());
		coupon.setRightName("代金券");
		coupon.setUsedTypeFlag(1);
		coupon.setTotalUseNum(-1);
		coupon.setId(1l);
		coupon.setSortValue(1);
		coupon.setStatus(1);
		coupon.setDescription("ces");
		coupon.setUseDescription("测试");
		Map<String,Integer> map = new HashMap<String,Integer>();
		map.put("price",10);
		coupon.setRightContent(map.toString());
		couponList.add(coupon);

		Coupon coupon1 = new Coupon();
		coupon1.setActiveEndTime(DateUtil.getTime());
		coupon1.setActiveStartTime(DateUtil.getTime());
		coupon1.setRightName("折扣券");
		coupon1.setUsedTypeFlag(2);
		coupon1.setTotalUseNum(-1);
		coupon1.setId(2l);
		coupon1.setSortValue(1);
		coupon1.setStatus(2);
		Map<String,Integer> map1= new HashMap<String,Integer>();
		map1.put("price",10);
		coupon1.setRightContent(map1.toString());
		coupon1.setDescription("ces");
		coupon1.setUseDescription("测试");
		couponList.add(coupon1);
		return couponList;
	}
}
